/** @file
===========================================================================
MODULE          : DirectShowAudioPipeline

DESCRIPTIONS    : 

REVISION HISTORY: Version 1.0

COPYRIGHT       : (c)CSIR, Meraka Institute 2007 all rights reserved

RESTRICTIONS	: The information/data/code contained within this file is 
				  the property of CSIR, Meraka Institute and has been 
				  classified as PUBLIC.
===========================================================================
*/
#include "StdAfx.h"
#include "DirectShowAudioPipeline.h"

#include <DirectShow/LiveIntegration/DirectShowSampleGrabber.h>

#include <Utils/DirectShowUtils/DirectShowHelper.h>
#include <Utils/StringConversion.h>

DirectShowAudioPipeline::DirectShowAudioPipeline(void)
	:m_pSource(NULL),
	m_pMediaType(NULL),
	m_sAudioDeviceName("")
{
	m_pAudioCB = new CDirectShowSampleGrabberCB(/*this*/ NULL);
}

DirectShowAudioPipeline::~DirectShowAudioPipeline(void)
{
	Stop();
	if (m_pMediaType)
		FreeMediaType(*m_pMediaType);

	if (m_pHeaderInfo)
		delete m_pHeaderInfo;
}
    
bool DirectShowAudioPipeline::InitDirectShow()
{
	HRESULT hr = m_pGraph.CreateInstance(CLSID_FilterGraph);
	if (SUCCEEDED(hr))
	{
		m_pControl = m_pGraph;
		return true;
	}
	else
	{
		return false;
	}
}

bool DirectShowAudioPipeline::Run()
{
	if (m_pControl)
	{
		HRESULT hr = m_pControl->Run();
		if (hr == S_FALSE)
		{
			// this is just background state change
			hr = S_OK;
		}
	}
	return true;
}

bool DirectShowAudioPipeline::Pause()
{
	if (m_pControl)
	{
		HRESULT hr = m_pControl->Pause();
		if (FAILED(hr))
		{
			return false;
		}
	}
	return true;
}

bool DirectShowAudioPipeline::Stop()
{
	if (m_pControl)
	{
		HRESULT hr = m_pControl->Stop();
		if (hr == S_FALSE)
		{
			// this is just background state change
			hr = S_OK;
		}
	}
	return true;
}

bool DirectShowAudioPipeline::LoadPipeLine()
{
	if (LoadFilters())
	{
		return ConnectFilters();
	}
	return false;
}

// This method retrieves the media type of the sample grabber for our callback
HRESULT DirectShowAudioPipeline::GetAudioMediaType( AM_MEDIA_TYPE** pMediaType )
{
	IPinPtr pPin;
	HRESULT hr = CDirectShowHelper::GetPin(m_pAudioSampleGrabber, PINDIR_INPUT, &pPin);
	if (SUCCEEDED(hr))
	{
		hr = pPin->ConnectionMediaType(*pMediaType);
		pPin = NULL;
		return hr;
	}
	pPin = NULL;
	return E_FAIL;  
}

void DirectShowAudioPipeline::SetLive555AudioSource( CDirectShowDeviceSource* pSource )
{
	m_pSource = pSource;
	m_pAudioCB->DirectShowDeviceSource(pSource);
}

bool DirectShowAudioPipeline::LoadFilters()
{
	if (m_sAudioDeviceName == "")
	{
		return false;
	}

	// Load the audio capture filter
	WCHAR* wszDeviceName = CStringConversion::ConvertStlToWide(m_sAudioDeviceName);
	//HRESULT hr = CDirectShowHelper::AddFilterByCategoryAndName(m_pGraph, CLSID_AudioInputDeviceCategory, L"Camera", &m_pAudioCaptureSource);
	HRESULT hr = CDirectShowHelper::AddFilterByCategoryAndName(m_pGraph, CLSID_AudioInputDeviceCategory, wszDeviceName, &m_pAudioCaptureSource);
	if (FAILED(hr))
	{
		delete[] wszDeviceName;
		return false;
	}
	delete[] wszDeviceName;

	hr = CDirectShowHelper::AddFilterByCLSID(m_pGraph, CLSID_ACMWrapper, L"ACM Wrapper", &m_pACMWrapper);
	if (FAILED(hr))
	{
		return false;
	}
	else
	{
		// !!! Commenting out for now
		// Set compression properties on ACM wrapper
		// 1) Enumerate output pins
		IPinPtr pOutputPin;
		hr = CDirectShowHelper::GetPin(m_pACMWrapper, PINDIR_OUTPUT, &pOutputPin);
		if (SUCCEEDED(hr))
		{
			m_pIamStreamConfigInterface = pOutputPin;

			if (!m_pIamStreamConfigInterface)
			{
				pOutputPin = NULL;
				return false;
			}
			// Now set the output pin properties
		}
		pOutputPin = NULL;
	}

	// Load the Sample Grabber
	hr = CDirectShowHelper::AddFilterByCLSID(m_pGraph, CLSID_SampleGrabber, L"Audio Sample Grabber", &m_pAudioSampleGrabber);
	// Do we need the WavDestFilter
	if (FAILED(hr))
	{
		return false;
	}
	else
	{
		// Configure Sample Grabber Callback
		m_pSampleGrabberInterface = m_pAudioSampleGrabber;
		m_pSampleGrabberInterface->SetCallback(m_pAudioCB, 0);
	}
	

	// Load the Null renderer
	hr = CDirectShowHelper::AddFilterByCLSID(m_pGraph, CLSID_NullRenderer, L"Audio NULL Renderer", &m_pAudioNullRenderer);
	if (FAILED(hr))
	{
		return false;
	}

	return true;
}

bool DirectShowAudioPipeline::ConnectFilters()
{
	// Connect capture source to compression filter
	HRESULT hr = CDirectShowHelper::ConnectFilters(m_pGraph, m_pAudioCaptureSource, m_pACMWrapper);
	if (FAILED(hr))
	{
		return false;
	}
	else
	{
		// First set compression filter media type
		int nCount, nSize;
		BYTE *pSCC = NULL;
		AM_MEDIA_TYPE *pmt;

		hr = m_pIamStreamConfigInterface->GetNumberOfCapabilities(&nCount, &nSize);

		pSCC = new BYTE[nSize];
		if (pSCC == NULL)
		{
			return false;
			// TODO: Out of memory error.
		}

		// Iterate over all formats until we find a 8 bit mono 8000
		for (int i = 0; i < nCount; i++)
		{
			// Get the first format.
			hr = m_pIamStreamConfigInterface->GetStreamCaps(i, &pmt, pSCC);
			if (hr == S_OK)
			{
				WAVEFORMATEX *pWF = (WAVEFORMATEX *) pmt->pbFormat;
				/*if ((pWF->nChannels == 1)&&
					(pWF->wBitsPerSample == 8)&&
					(pWF->nSamplesPerSec == 8000))*/
				if ((pWF->nChannels == 1)&&
					(pWF->wBitsPerSample == 16)&&
					(pWF->nSamplesPerSec == 8000))
				{
					// Set the format
					hr = m_pIamStreamConfigInterface->SetFormat(pmt);
					if (FAILED(hr))
					{
						// TODO: Error handling.
						return false;
					}
				}
				DeleteMediaType(pmt);
			}
		}

		// Now free memory
		delete [] pSCC;
	}

	// Connect all the filters
	hr = CDirectShowHelper::ConnectFilters(m_pGraph, m_pACMWrapper, m_pAudioSampleGrabber);
	if (FAILED(hr))
	{
		return false;
	}
	else
	{
		/************************/

		// Get the media type of the audio pipeline
		m_pMediaType = new AM_MEDIA_TYPE;
		hr = GetAudioMediaType(&m_pMediaType);
		if(SUCCEEDED(hr))
		{
			// 4/02/2008 Testing if I can extract all data from WAVEFORMATEX structure rather than recreating a header and parsing it
			if ((m_pMediaType->majortype == MEDIATYPE_Audio)&&(m_pMediaType->subtype == MEDIASUBTYPE_PCM))
			{
				WAVEFORMATEX* pWavHeader = (WAVEFORMATEX*)m_pMediaType->pbFormat;
				m_pHeaderInfo = new WavHeaderData(pWavHeader->wFormatTag, pWavHeader->wBitsPerSample, pWavHeader->nSamplesPerSec, pWavHeader->nChannels);
				/*
				WANT THE FOLLOWING:
				unsigned char fAudioFormat;
				unsigned char fBitsPerSample;
				unsigned fSamplingFrequency;
				unsigned fNumChannels;
				unsigned fWAVHeaderSize;
				double fPlayTimePerSample;
				unsigned fPreferredFrameSize;
				*/
			}
		}
	}

	hr = CDirectShowHelper::ConnectFilters(m_pGraph, m_pAudioSampleGrabber, m_pAudioNullRenderer);
	if (FAILED(hr))
	{
		return false;
	}
	
	return true;
}