/** @file

MODULE						: PicCropperBase

TAG							: PCB

FILE NAME					: PicCropperBase.h

DESCRIPTION					: This class is the base class defining the minimum interface	
							  and properties for all derived implementations to crop an
							  image and write it to another image. The instantiation process 
							  must set the parameters and limits to the images such that
							  the Crop() implementation only requires source image data
							  pointers. The insertion method is pure virtual and must be 
							  implemented by derived classes.

REVISION HISTORY			: 03/10/2007 KF - the parameters of the Crop() method
							  are reversed so input to output is left to right.

COPYRIGHT					: (c)CSIR, Meraka Institute 2007 all rights reserved

RESTRICTIONS				: The information/data/code contained within this file is 
							  the property of CSIR, Meraka Institute and has been 
							  classified as PUBLIC.
===========================================================================
*/
#ifndef _PICCROPPERBASE_H
#define _PICCROPPERBASE_H

#pragma once

/*
===========================================================================
  Class definition.
===========================================================================
*/
class PicCropperBase
{
public:
	// Construction and destruction.
	PicCropperBase(void) {_widthOut = 0; _heightOut = 0; _widthIn = 0; _heightIn = 0; _byX1 = 0; _byX2 = 0; _byY1 = 0; _byY2 = 0; }
	PicCropperBase(int widthOut, int heightOut, int widthIn, int heightIn, int left, int right, int top, int bottom) 
	{	_widthOut = widthOut; _heightOut = heightOut; _widthIn = widthIn; _heightIn = heightIn; _byX1 = left; _byX2 = right; _byY2 = top; _byY1 = bottom; }
	virtual ~PicCropperBase(void) {}

	// Interface.
	virtual int Crop(void* pInImg, void* pImg) = 0;

	// Member interface.
	int	GetOutWidth(void)		{ return(_widthOut); }
	int	GetOutHeight(void)	{ return(_heightOut); }
	int	GetInWidth(void)		{ return(_widthIn); }
	int	GetInHeight(void)		{ return(_heightIn); }
	int	GetCropLeft(void)		{ return(_byX1); }
	int	GetCropRight(void)	{ return(_byX2); }
	int	GetCropBottom(void)	{ return(_byY1); }
	int	GetCropTop(void)		{ return(_byY2); }
  
	void SetOutDimensions(int widthOut, int heightOut)			{_widthOut = widthOut; _heightOut = heightOut; }
	void SetInDimensions(int widthIn, int heightIn)					{_widthIn = widthIn; _heightIn = heightIn; }
	void SetCrop(int left, int right, int top, int bottom)	{_byX1 = left; _byX2 = right; _byY2 = top; _byY1 = bottom; }

protected:
	// Members.
	int	_widthOut;		/// Of cropped output image.
	int	_heightOut;
	int _widthIn;			/// Of input image to crop.
	int _heightIn;
	int _byX1;				/// Crop pels from the left.
	int _byX2;				/// Crop pels from the right.
	int _byY1;				/// Crop from the bottom.
	int _byY2;				/// Crop from the top.

};//end PicCropperBase.

#endif	// _PICCROPPERBASE_H
